import logging
from decimal import Decimal

NUMBER_TYPES = (int, float, Decimal)

__version__ = "0.0.2"
__name__ = "imagery-mapilio"
__major__ = False
__minor__ = False
__patch__ = True

logger = logging.getLogger('imagery')


